# coding: utf-8

from .base import BasePattern


class CyclePattern(BasePattern):
    """
    select an element regularly from the choices.
    """
    def __init__(self, choices, increment=1, interval=1, callback=None):
        """
        :param choices: elements of choices <list> or <dict> (element(key): element_count(value))
        :param increment: list-index increase. <int>
        :param interval: intervals up proceeds to the next element. <int>
        :param callback: function to be called at the end of rendering. <function>
        :return: Cycle Pattern instance.
        """
        assert interval > 0
        if isinstance(choices, dict):
            self.choices = []
            for value, num in choices.items():
                self.choices += [value] * num
        else:
            self.choices = choices
        self.size = len(self.choices)
        self.index = 0
        self.count = 0
        self.interval = interval
        self.increment = increment
        self.add_callback(callback)

    def render(self, *args):
        """
        render a element by regularly selection.
        :param args: any.
        :return: rendered element.
        """
        element = self.choices[self.index % self.size]

        self.count += 1
        if self.count % self.interval == 0:
            self.index += self.increment
        for callback in self.callbacks:
            element = callback(element)
        return element
