# coding: utf-8

from .base import BasePattern


class SequencePattern(BasePattern):
    """
    select an element sequentially.
    """
    def __init__(self, sequence, missing=None, interval=1, callback=None):
        """
        :param sequence: elements of sequence <list> or <dict> (element(key): element_count(value))
        :param missing: return this value if reached last element. <any>
        :param callback: function to be called at the end of rendering. <function>
        :return: Choice Pattern instance.
        """
        assert interval > 0
        if isinstance(sequence, dict):
            self.sequence = []
            for value, num in sequence.items():
                self.sequence += [value] * num
        else:
            self.sequence = sequence
        self.size = len(self.sequence) - 1
        self.missing = missing
        self.index = 0
        self.count = 0
        self.interval = interval
        self.add_callback(callback)

    def render(self, *args):
        """
        render a element
        :param args: any.
        :return: rendered element.
        """
        if self.index > self.size:
            return self.missing

        element = self.sequence[self.index]
        self.count += 1
        if self.count % self.interval == 0:
            self.index += 1

        for callback in self.callbacks:
            element = callback(element)
        return element
